"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveToLessons = exports.moveToLearned = exports.moveToUnknown = exports.addToLessons = exports.addToLearned = exports.addToUnknown = exports.getCardsForCurrentLesson = exports.getCardInfo = exports.getCardsForLesson = exports.isLastLessonForCard = exports.setCurrentLesson = exports.getCurrentLesson = exports.createLeitnerBox = void 0;
const R = __importStar(require("ramda"));
const utils_1 = require("./utils");
const unknownLens = R.lensPath(['decks', 'unknown']);
const learnedLens = R.lensPath(['decks', 'learned']);
const lessonsLens = (index) => R.lensPath(['decks', 'lessons', index, 'cards']);
const addCardTo = (cardLens, box, card) => {
    return card
        ? R.over(cardLens, R.append(card), box)
        : box;
};
const findAndRemove = (findLens, updateLens, identity, box) => {
    const card = R.view(findLens, box);
    return card ? [R.over(updateLens, R.reject(identity), box), card] : null; // Return null when the card is not found
};
const findAndRemoveInLessons = (findLens, updateLens, identity, box) => {
    for (let i = 0; i < box.decks.lessons.length; i++) {
        const result = findAndRemove(findLens(i), updateLens(i), identity, box);
        if (result) {
            return result;
        }
    }
    return null;
};
const removeCard = (box, identity) => {
    const findInUnknownLens = R.compose(unknownLens, (0, utils_1.lensMatchIdentity)(identity));
    const findInLearnedLens = R.compose(learnedLens, (0, utils_1.lensMatchIdentity)(identity));
    const findInLessonsLens = (index) => R.compose(lessonsLens(index), (0, utils_1.lensMatchIdentity)(identity));
    const result = findAndRemove(findInUnknownLens, unknownLens, identity, box) ||
        findAndRemove(findInLearnedLens, learnedLens, identity, box) ||
        findAndRemoveInLessons(findInLessonsLens, lessonsLens, identity, box);
    return result || [box, null];
};
const createLeitnerBox = ({ repetitions = 3, currentLesson = 0, initialDecks } = {}) => {
    const count = (0, utils_1.fib)(repetitions) + 1;
    const repeatOn = (0, utils_1.buildRepeatLessons)(count, repetitions);
    const rawWorkingDecks = initialDecks || (0, utils_1.buildEmptyDecks)(count);
    (0, utils_1.assert)(rawWorkingDecks.length === count + 2, `Initial decks number doesn't fit repetitions count. Must be ${count + 2}`);
    const decks = {
        unknown: rawWorkingDecks[0],
        learned: rawWorkingDecks[rawWorkingDecks.length - 1],
        lessons: rawWorkingDecks.slice(1, -1).map((deck, index) => ({
            repeatOn: repeatOn[index],
            cards: deck
        }))
    };
    return {
        count,
        repetitions,
        currentLesson,
        decks
    };
};
exports.createLeitnerBox = createLeitnerBox;
const getCurrentLesson = (box) => box.currentLesson;
exports.getCurrentLesson = getCurrentLesson;
const setCurrentLesson = (box, currentLesson) => {
    const currentLessonLens = R.lensProp('currentLesson');
    return R.set(currentLessonLens, currentLesson, box);
};
exports.setCurrentLesson = setCurrentLesson;
const isLastLessonForCard = (box, identity) => {
    const { cards } = box.decks.lessons.find(({ repeatOn }) => repeatOn[repeatOn.length - 1] === box.currentLesson);
    return cards.some(identity);
};
exports.isLastLessonForCard = isLastLessonForCard;
const getCardsForLesson = (box, lesson) => {
    return box.decks.lessons.reduce((result, { repeatOn, cards }) => repeatOn.includes(lesson) ? [...result, ...cards] : result, []);
};
exports.getCardsForLesson = getCardsForLesson;
const getCardInfo = (box, identity) => {
    const findInUnknownLens = R.compose(unknownLens, (0, utils_1.lensMatchIdentity)(identity));
    const findInLearnedLens = R.compose(learnedLens, (0, utils_1.lensMatchIdentity)(identity));
    if (R.view(findInUnknownLens, box)) {
        return {
            deck: 'unknown'
        };
    }
    if (R.view(findInLearnedLens, box)) {
        return {
            deck: 'learned'
        };
    }
    const currentLesson = (0, exports.getCurrentLesson)(box);
    for (let i = 0; i < box.decks.lessons.length; i++) {
        const { repeatOn, cards } = box.decks.lessons[i];
        const cardIndex = cards.findIndex(identity);
        if (cardIndex !== -1) {
            return {
                deck: 'lessons',
                repetitionsLeft: repeatOn.slice(repeatOn.findIndex((l) => l >= currentLesson)).length
            };
        }
    }
    return {
        deck: 'undefined'
    };
};
exports.getCardInfo = getCardInfo;
const getCardsForCurrentLesson = (box) => (0, exports.getCardsForLesson)(box, box.currentLesson);
exports.getCardsForCurrentLesson = getCardsForCurrentLesson;
exports.addToUnknown = R.partial(addCardTo, [unknownLens]);
exports.addToLearned = R.partial(addCardTo, [learnedLens]);
const addToLessons = (box, card) => addCardTo(lessonsLens(box.currentLesson), box, card);
exports.addToLessons = addToLessons;
exports.moveToUnknown = R.pipe(removeCard, R.apply(exports.addToUnknown));
exports.moveToLearned = R.pipe(removeCard, R.apply(exports.addToLearned));
exports.moveToLessons = R.pipe(removeCard, R.apply(exports.addToLessons));
